//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Class to display the mesh of a data object
//* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
class DataMesh{
    
    Colormap _colormap;
    color [] _meshColors;
    Point _origin;

    // * * * Constructor (default) * * * //
    DataMesh(Colormap colormap) { _colormap = colormap; _origin = new Point(); }

    // * * * Constructor that sets the colormap * * * //
    DataMesh() { this(new RainbowNegValueColormapFactory().getColormap()); }
    
    // * * * Set/Get colormap * * * //
    void setColormap(Colormap colormap)    { _colormap = colormap; }
    Colormap getColormap()                 { return _colormap; }
    TwoDimensionalColormap getColormap2D() { return (TwoDimensionalColormap)_colormap; }
   
    // * * * Set/Get the origin * * * //
    void setOrigin(Point o){ _origin = o; }
    Point getOrigin(){ return _origin; }

    // * * * Set the colormap data * * * //
    void setColorData(double [] data, double min, double max){
	_colormap.setRange((float)min, (float)max);
	_meshColors = new color[data.length];
	for(int i = 0; i < data.length; i++)
	    _meshColors[i] = _colormap.getColor((float)data[i]);
    }

    // * * * Set the colormap data (2D) * * * //
    void setColorData(double[] data1, double [] data2, 
		      double min1, double max1, 
		      double min2, double max2){
	((TwoDimensionalColormap)_colormap).setRange((float)min1, (float)max1, 
						     (float)min2, (float)max2);
	_meshColors = new color[data1.length];
	for(int i = 0; i < data1.length; i++)
	    _meshColors[i] = ((TwoDimensionalColormap)_colormap).getColor((float)data1[i], 
									  (float)data2[i]);
    }
    
    // * * * Draw the mesh * * * //
    void drawMesh(DataObject object, boolean drawMesh, Point trans, Point rot) {

	// If we have a mesh, draw it
	if(object.getNumPts() != -1) {
	 
	    // Do the 3D user transformations
	    pushMatrix();
	    translate(width/2, height/2, 0);
	    translate(_origin.x(), _origin.y(), _origin.z());
	    translate(trans.x(), trans.y(), trans.z());
	    rotateX(radians(rot.x()));
	    rotateY(radians(rot.y()));

	    // Draw the surface
	    noStroke();
	    beginShape(TRIANGLES);
	    fill(255, 0, 0);
	    for(int i = 0; i < object.getNumTris(); i++) {
		Triangle t = object.getConnect(i);
		fill(_meshColors[t.a()]);
		setVertex(object.getGeometry(t.a()) );
		fill(_meshColors[t.b()]);
		setVertex(object.getGeometry(t.b()) );
		fill(_meshColors[t.c()]);
		setVertex(object.getGeometry(t.c()));
	    }	
	    endShape();
	    
	    if(drawMesh){
		// Draw the wireframe
		noFill(); strokeWeight(1);
		beginShape(TRIANGLES);
		for(int i = 0; i < object.getNumTris(); i++) {
		    Triangle t = object.getConnect(i);
		    stroke(dimColor(_meshColors[t.a()]));
		    offVertex( object.getGeometry(t.a()));
		    stroke(dimColor(_meshColors[t.b()]));
		    offVertex(object.getGeometry(t.b()));
		    stroke(dimColor(_meshColors[t.b()]));
		    offVertex(object.getGeometry(t.c()));
		}
		endShape();
	    }
	    popMatrix();
	}
    }
}

// * * * Shortcut to set the vertex * * * //
void setVertex(Point pt){
    vertex(pt.x(), pt.y(), pt.z()-0.1);
}

// * * * Shortcut to set the vertex with an offset * * * //
void offVertex(Point pt){
    vertex(pt.x(), pt.y(), pt.z());
}

// * * * Shortcut to dim a color * * * //
color dimColor(color col){
    float dim = .75;
    return color(red(col)*dim, green(col)*dim, blue(col)*dim, 125);
}
